package fi.jyu.mit.graphics;

import java.awt.*;
import javax.swing.*;

/**
 * Luokka jonka avulla voidaan helposti luoda selaimella suoritettavia sovelmia
 * @author Markus Kivioja
 *
 */
public class EasyApplet extends JApplet {

    private final SpacePanel panel;
    private TouchPad tp;
    private ControlButtons cb;
    private static final long serialVersionUID = 1L;
    private final JPanel tpPanel = new JPanel();
    private final JPanel cbPanel = new JPanel();
    private final Panel controlpanel = new Panel();


    /**
     * Luo uuden sovelman jonka koko on 400x550
     *
     */
    public EasyApplet() {
        super();
        Container pane = getContentPane();
        pane.setLayout(new BorderLayout());
        panel = new SpacePanel();
        panel.setBounds(0, 0, 400, 300);
        pane.add(panel, BorderLayout.CENTER);
        pane.setBounds(0, 0, 400, 550);
        this.setBounds(0, 0, 400, 550);
        pane.setPreferredSize(new Dimension(400, 550));
        pane.setBackground(Color.WHITE);
        controlpanel.setLayout(new BorderLayout());
        pane.add(controlpanel, BorderLayout.SOUTH);
        tpPanel.setVisible(false);
        cbPanel.setVisible(false);
        controlpanel.add(tpPanel, BorderLayout.WEST);
        controlpanel.add(cbPanel, BorderLayout.EAST);
        tpPanel.setBorder(BorderFactory.createLineBorder(Color.black));
        cbPanel.setBorder(BorderFactory.createLineBorder(Color.black));
    }


    /**
     * Asettaa sovelmaan kosketusohjaimen
     * @param touchPadOn kosketusohjain nkyvill, tosi/eptosi
     * @param move tarviitseeko hiiren painikkeen olla painettuna, tosi/eptosi
     */
    public void setTouchPad(boolean touchPadOn, boolean move) {
        if (touchPadOn) {
            if (tp == null) {
                tp = new TouchPad(panel, move);
                tpPanel.add(tp);
                tp.setPreferredSize(new Dimension(160, 150));
            } else
                tp.setMove(move);
        }
        tpPanel.setVisible(touchPadOn);
    }


    /**
     * Asettaa sovelmaan sellaisen kosketusohjaimen jossa hiiren painikkeen
     * ei tarvitse olla painettuna
     * @param touchPadOn kosketusohjain nkyvill, tosi/eptosi
     */
    public void setTouchPad(boolean touchPadOn) {
        this.setTouchPad(touchPadOn, false);
    }


    /**
     * Asettaa sovelmaan ohjainpainikket
     * @param controlButtonsOn ohjainpainikkeet nkyvill, tosi/eptosi
     */
    public void setControlButtons(boolean controlButtonsOn) {
        if (controlButtonsOn && (cb == null)) {
            cb = new ControlButtons(panel);
            cbPanel.add(cb);
            cb.setPreferredSize(new Dimension(160, 150));
        }
        cbPanel.setVisible(controlButtonsOn);
    }


    /**
     * Palauttaa paneelin jonka tm sovelma nytt
     * @return paneeli jonka tm sovelma nytt
     */
    public SpacePanel getPanel() {
        return this.panel;
    }


    /**
     * Palauttaa paneelin joka sislt olioiden ohjaimet
     * @return paneeli joka sislt olioiden ohjaimet
     */
    public Panel getControlPanel() {
        return this.controlpanel;
    }


    /**
     * Lis sovelmaan uuden olio-ohjaimen
     * @param controller listtv olio-ohjain
     */
    public void add(BasicController controller) {
        controlpanel.setLayout(new FlowLayout());
        JPanel panel = new JPanel();
        panel.setVisible(true);
        panel.add(controller);
        controller.setPreferredSize(new Dimension(160, 150));
        controlpanel.add(panel, BorderLayout.WEST);
        panel.setBorder(BorderFactory.createLineBorder(Color.black));

    }


    /**
     * Korvaa tm perityiss luokissa
     * @see java.applet.Applet#init()
     */
    @Override
    public void init() {
        // Tm pit korvata perityss luokassa
    }
}
