package fi.jyu.mit.barChart;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.*;
import fi.jyu.mit.graphics.*;

/**
 * Pylvsdiagrammikomponentti. Silytt nyttmns datan.
 * @author Markus Kivioja
 * @version 25.05.09
 */
public class BarChart extends JPanel {

    private static final int DEF_POINTS = 20;

    private SpacePanel s = new SpacePanel();
    private static final long serialVersionUID = 0;
    private double marginal = 0.15;
    private BarListener barListener;
    private FillPolygon background = new FillPolygon();
    private DrawableCollection bars = new DrawableCollection();
    private Axis axis = new Axis();
    private RPoint[] screenCorners = { new RPoint(), new RPoint(), new RPoint(),
            new RPoint() };
    private Vector temp = new Vector();

    // Properties
    private boolean showXscale = true;
    private boolean showYscale = true;
    private Color backgroundColor = Color.lightGray;
    private Color barColor = Color.yellow;
    private Color axisColor = Color.black;
    private int xScalePoints = 5;
    private int yScalePoints = 5;
    private double maxY = 100;
    private int points = 0;


    /**
     * Luo uuden BarChart-komponentin
     */
    public BarChart() {
        try {
            init();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }


    /**
     * Alustaa komponentin asetukset
     * @throws Exception
     */
    private void init() throws Exception {
        this.setPoints(DEF_POINTS);
        this.scale();
        this.axis.showXscale(this.showXscale);
        this.axis.showYscale(this.showYscale);
        this.axis.setXscalePoints(this.xScalePoints * 2);
        this.axis.setYscalePoints(this.yScalePoints * 2);
        this.setLayout(new BorderLayout());
        this.add(s, BorderLayout.CENTER);

        this.s.setIsotrophic(false);
        this.s.add(this.background).setColor(this.backgroundColor);
        this.s.add(this.bars).setColor(this.barColor);
        this.s.add(this.axis);

        this.addMouseListener(new MouseAdapter() {
            @SuppressWarnings("synthetic-access")
            @Override
            public void mouseClicked(final MouseEvent e) {
                if (barListener == null)
                    return;
                temp.set((double) e.getX(), (double) e.getY());
                Vector transformed = s.getFullTransform().getInversion()
                        .multiply(temp);
                int i = (int) transformed.a(0);
                double y = transformed.a(1);
                if (i < 0)
                    i = 0;
                if (i > points - 1)
                    i = points - 1;
                if (y < 0)
                    y = 0;
                if (y > maxY)
                    y = maxY;
                barListener.barClicked(new BarEvent((Bar) bars.getDrawable(i),
                        BarEvent.BAR_CLICKED, y));
            }
        });
    }


    /**
     * Skaalaa ikkunan oikean kokoiseksi
     */
    public void scale() {
        this.screenCorners[0].set(-this.points * marginal,
                -this.maxY * marginal);
        this.screenCorners[1].set(-this.points * marginal,
                this.maxY * (1 + marginal));
        this.screenCorners[2].set(this.points * (1 + marginal),
                this.maxY * (1 + marginal));
        this.screenCorners[3].set(this.points * (1 + marginal),
                -this.maxY * marginal);
        this.background.setPoints(this.screenCorners);
        this.s.scale(-this.points * marginal, -this.maxY * marginal,
                this.points * (1 + marginal), this.maxY * (1 + marginal));
        this.axis.setLengths(this.points * 2, this.maxY * 2, 0);
    }


    /**
     * Asettaa akselien varin
     * @param color akselien vari
     */
    public void setAxisColor(Color color) {
        this.axisColor = color;
        this.axis.setColor(color);
    }


    /**
     * Asettaa x-akselin asteikon nkyvksi
     * @param showXscale asteikko nkyv/piilotettu
     */
    public void setShowXscale(boolean showXscale) {
        this.showXscale = showXscale;
        this.axis.showXscale(showXscale);
    }


    /**
     * Asettaa y-akselin asteikon nkyvksi
     * @param showYscale asteikko nkyv/piilotettu
     */
    public void setShowYscale(boolean showYscale) {
        this.showYscale = showYscale;
        this.axis.showYscale(showYscale);
    }


    /**
     * Asettaa x-akselin asteikon lukujen mrn
     * @param xScalePoints asteikon lukujen mr
     */
    public void setXScalePoints(int xScalePoints) {
        this.xScalePoints = xScalePoints;
        this.axis.setXscalePoints(xScalePoints * 2);
    }


    /**
     * Asettaa y-akselin asteikon lukujen mrn
     * @param yScalePoints asteikon lukujen mr
     */
    public void setYScalePoints(int yScalePoints) {
        this.yScalePoints = yScalePoints;
        this.axis.setYscalePoints(yScalePoints * 2);
    }


    /**
     * Asettaa diagrammin korkeuden
     * @param maxY diagrammin korkeus
     */
    public void setMaxY(double maxY) {
        this.maxY = maxY;
        this.scale();
    }


    /**
     * Asettaa pylviden vrin
     * @param barColor pylviden vri
     */
    public void setBarColor(Color barColor) {
        this.barColor = barColor;
        this.bars.setColor(this.barColor);
    }


    /**
     * Asettaa taustavrin
     * @param backgroundColor taustavri
     */
    public void setBackgroundColor(Color backgroundColor) {
        this.backgroundColor = backgroundColor;
        this.setBackground(this.backgroundColor);
        this.background.setColor(this.backgroundColor);
    }


    /**
     * Asettaa pylviden maksimimrn
     * @param points pylviden maksimimr
     */
    public void setPoints(int points) {
        if (points < this.points) {
            for (int i = this.points - 1; i >= points; i--)
                this.bars.remove(this.bars.getDrawable(i));
            this.points = points;
            this.scale();
            return;
        }
        for (int i = this.points; i < points; i++)
            this.bars.add(new Bar(i, 1, 0));
        this.points = points;
        this.scale();
    }


    /**
     * Asettaa halutun pylvn korkeuden
     * @param i pylvn paikka diagrammissa
     * @param data pylvn korkeus
     */
    public void setData(int i, double data) {
        if (this.bars.getDrawable(i) == null)
            return;
        ((Bar) this.bars.getDrawable(i)).setValue(data);
    }


    /**
     * Kertoo onko x-akselin asteikko nkyvill
     * @return asteikko nkyvill/piilotettu
     */
    public boolean getShowXscale() {
        return this.showXscale;
    }


    /**
     * Kertoo onko y-akselin asteikko nkyvill
     * @return asteikko nkyvill/piilotettu
     */
    public boolean getShowYscale() {
        return this.showYscale;
    }


    /**
     * Antaa x-akselin asteikon lukujen mrn
     * @return x-akselin asteikon lukujen mr
     */
    public int getXScalePoints() {
        return this.xScalePoints;
    }


    /**
     * Antaa y-akselin asteikon lukujen mrn
     * @return asteikon lukujen mr
     */
    public int getYScalePoints() {
        return this.yScalePoints;
    }


    /**
     * Antaa diagrammin korkeuden
     * @return diagrammin korkeus
     */
    public double getMaxY() {
        return this.maxY;
    }


    /**
     * Antaa pylviden maksimimrn
     * @return pylviden maksimimr
     */
    public int getPoints() {
        return this.points;
    }


    /**
     * Antaa halutun pylvn korkeuden
     * @param i pylvn paikka diagrammissa
     * @return pylvn korkeus
     */
    public double getData(int i) {
        return getBar(i).getValue();
    }


    /**
     * @param i moneko pylvs halutaan
     * @return i:s pylvs
     */
    public Bar getBar(int i) {
        return (Bar) bars.getDrawable(i);
    }


    /**
     * Antaa pylviden vrin
     * @return pylviden vri
     */
    public Color getBarColor() {
        return this.barColor;
    }


    /**
     * Antaa taustavrin
     * @return taustavri
     */
    public Color getBackgroundColor() {
        return this.backgroundColor;
    }


    /**
     * Antaa akselien vrin
     * @return akselein vri
     */
    public Color getAxisColor() {
        return this.axisColor;
    }


    /**
     * Rekisteröi uuden BarEvent-tapahtumien kuuntelijan
     * @param l tapahtumakuuntelija
     */
    public synchronized void addBarListener(BarListener l) {
        barListener = BarEventMulticaster.add(barListener, l);
    }


    /**
     * Poistaa BarEvent-tapahtumien kuuntelijan
     * @param l tapahtumakuuntelija
     */
    public synchronized void removeBarListener(BarListener l) {
        barListener = BarEventMulticaster.remove(barListener, l);
    }
}