package fi.jyu.mit.graphics;

import java.util.Timer;
import java.util.TimerTask;

/**
 * Kantaluokka yleisille ajastetuille muuntimille
 * @author Vesa Lappalainen
 */
public class BasicTransformer extends TimerTask implements Transformer {

    private final Transformable object;
    private Matrix tra;
    private final Timer timer;


    /**
     * Luo uuden liikuttajan
     * @param object olio jota liikutetaan
     * @param tra muunosmatriisi
     * @param dt muunnos askelien vlinen aika millisekunteina
     */
    public BasicTransformer(Transformable object, Matrix tra, long dt) {
        super();
        this.object = object;
        this.tra = tra;
        timer = new Timer();
        timer.schedule(this, dt, dt);
    }


    /**
     * Kynnist liikuttajan
     */
    @Override
    public void run() {
        object.transform(tra);
    }


    /**
     * Asettaa muuntelijalle uuden muunnosmatriisin
     * @param rot uusi kiertomatriisi
     */
    public void setTransform(Matrix rot) {
        this.tra = rot;
    }


    /**
     * @return nykyinen muunnosmatriisi
     */
    public Matrix getTransform() {
        return tra;
    }


    /**
     * Poistaa muuntelijan
     */
    @Override
    public void remove() {
        this.cancel();
        timer.cancel();
    }


    /**
     * @return muunneltava olio
     */
    public Transformable getObject() {
        return object;
    }

}
