package fi.jyu.mit.graphics;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Ohjain jolla voi kiert olioita akseliensa ympri,
 * painike jokaiselle akselille molempiin suuntiin
 * @author Markus Kivioja
 *
 */
public class ControlButtons extends BasicController implements ActionListener {

    private static final long serialVersionUID = 1L;

    protected boolean autoOn = false;
    private Rotator autoRotate;
    protected Matrix lastRot = null;
    private final JPanel xPanel = new JPanel();
    private final JPanel yPanel = new JPanel();
    private final JPanel zPanel = new JPanel();
    private final JPanel autoPanel = new JPanel();
    private final JCheckBox auto = new JCheckBox("Auto");


    /**
     * Luo uuden painikeohjaimen
     * @param objectToControl olio jota tll ohjaimella ohjataan
     */
    public ControlButtons(Transformable objectToControl) {
        super(objectToControl);
        this.setBounds(150, 350, 150, 150);

        this.xPanel.setLayout(new BorderLayout());
        this.yPanel.setLayout(new BorderLayout());
        this.zPanel.setLayout(new BorderLayout());
        this.autoPanel.setLayout(new BorderLayout());
        this.setLayout(new FlowLayout());

        this.autoPanel.add(auto, BorderLayout.CENTER);

        new TransformButton("KierraX").set(new RotMatrix(Axis.X, -4), xPanel,
                BorderLayout.WEST, this);
        new TransformButton("KierraX").set(new RotMatrix(Axis.X, 4), xPanel,
                BorderLayout.EAST, this);
        new TransformButton("KierraY").set(new RotMatrix(Axis.Y, -4), yPanel,
                BorderLayout.WEST, this);
        new TransformButton("KierraY").set(new RotMatrix(Axis.Y, 4), yPanel,
                BorderLayout.EAST, this);
        new TransformButton("KierraZ").set(new RotMatrix(Axis.Z, -4), zPanel,
                BorderLayout.WEST, this);
        new TransformButton("KierraZ").set(new RotMatrix(Axis.Z, 4), zPanel,
                BorderLayout.EAST, this);

        this.auto.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent tap) {
                if (tap.getStateChange() == ItemEvent.SELECTED) {
                    autoOn = true;
                    if (lastRot != null)
                        setAutoRotate(new Rotator(getObject(), lastRot, 20));
                }
                if (tap.getStateChange() == ItemEvent.DESELECTED) {
                    autoOn = false;
                    if (getAutoRotate() != null)
                        getAutoRotate().cancel();
                }
            }
        });

        add(xPanel);
        add(yPanel);
        add(zPanel);
        add(autoPanel);
    }


    /**
     * Tapahtumanksittelij joka ksittelee painikkeiden painallukset
     * @param e ksiteltv tapahtuma
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        Object causer = e.getSource();
        if (!(causer instanceof TransformButton))
            return;
        rotate(((TransformButton) causer).getTransform());
    }


    /**
     * Kiert tmn ohjaimen ohjaamaa oliota annetulla kiertomatriisilla
     * @param rotM kiertomatriisi
     */
    private void rotate(Matrix rotM) {
        lastRot = rotM;
        if (autoOn) {
            if (getAutoRotate() != null)
                getAutoRotate().cancel();
            setAutoRotate(new Rotator(getObject(), rotM, 20));
        } else
            // getObject().transform(rotM);
            getObject().setRotator(getObject().getRotator().multiply(rotM));
    }


    public Rotator getAutoRotate() {
        return autoRotate;
    }


    public void setAutoRotate(Rotator autoRotate) {
        this.autoRotate = autoRotate;
    }
}
