package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Rajapinta kaikille piirtokuvioille
 * @author Markus Kivioja
 *
 */
public interface Drawable extends Transformable {

    /**
     * Piirt kuvion annetulla Graphics-oliolla ja muunnosmatriisilla
     * @param g Graphics-olio
     * @param A muunnosmatriisi
     */
    void draw(Graphics g, Matrix A);


    /**
     * Asettaa kuvion vrin annetuilla punaisen, vihren ja sinisen arvoilla
     * ja palauttaa kuvion itsens
     * @param r vrin punaisen arvo
     * @param g vrin vihren arvo
     * @param b vrin sinisen arvo
     * @return kuvio
     */
    Drawable setColor(int r, int g, int b);


    /**
     * Asettaa kuvion vrin annetuksi Color-olioksi ja paluttaa kuvion itsens
     * @param c vri Color-oliona
     * @return kuvio
     */
    Drawable setColor(Color c);


    /**
     * Palautetaan vri jota kyten piirtmiseen
     * @return kuvion vri
     */
    public Color getColor();


    /**
     * Asettaa kuvion vanhemmat
     * @param parent joukko kuvion vanhemmista
     */
    void setParent(BasicDrawableCollection parent);


    /**
     * Piirt kuvion uudelleen
     *
     */
    void redraw();


    /**
     * Muuntaa kuvion muunosmatriisilla m
     * @param m matriisi jolla olion nykyinen muunnosmatrisi kerrotaan
     * @return muutettu matriisi
     */
    @Override
    Drawable transform(Matrix m);


    /**
     * Asettaa kuvion uudeksi muunnosmatriisiksi annetun matriisin
     * @param m uusi muunnosmatriisi
     * @return palauttaa muunnetun olion
     */
    Drawable setTransform(Matrix m);


    /**
     * Vaihtaa kuvion muunnosmatriisin annetuksi matriisiksi
     * @param m uusi muunnosmatriisi
     */
    @Override
    void changeTransform(Matrix m);


    /**
     * Skaalaa kuvion annettujen kertoimien mukaan
     * <pre>
     *      shape.scale(2, 1, 1); venytt kuviota kaksinkertaiseksi x-suunnassa
     * </pre>
     * @param dx kuvion x-suuntainen kerroin
     * @param dy kuvion y-suuntainen kerroin
     * @param dz kuvion z-suuntainen kerroin
     * @return palauttaa muunnetun olion
     */
    Drawable scale(double dx, double dy, double dz);


    /**
     * Liikuttaa kuviota, esimerkiksi:
     * <pre>
     *     shape.move(0, 1.5, -15);
     * </pre>
     * @param sx liikutettava matka x-akselin suunnassa
     * @param sy liikutettava matka y-akselin suunnassa
     * @param sz liikutettava matka z-akselin suunnassa
     * @return palauttaa muunnetun olion
     */
    Drawable move(double sx, double sy, double sz);


    /**
     * Kiert kuviota annetun akselin ympri, esimerkiksi:
     * <pre>
     *     shape.rotate(Axis.X, 45.2);
     * </pre>
     * @param axis akseli jonka ympri pyritetn (Axis.X/Axis.Y/Axis.Z)
     * @param deg kierrettvn kulman suuruus asteina
     * @return palauttaa muunnetun olion
     */
    Drawable rotate(int axis, double deg);

}
