package fi.jyu.mit.graphics;

/**
 * Rajapinta "helpolle piirtmiselle".
 * 
 * @author Markus Kivioja
 * 
 */
public interface Easy {

    /**
     * Lis olioon uuden viivan. Palauttaa viitteen listtyyn
     * viivaan, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *     space.addLine(0,0,100,100).setColor(Color.RED);
     * </pre>
     * @param x1 viivan alkupisteen x
     * @param y1 viivan alkupisteen y
     * @param x2 viivan loppupisteen x
     * @param y2 viivan loppupisteen y
     * @return listtyn viivaolion viite
     */
    Line addLine(double x1, double y1, double x2, double y2);


    /**
     * Lis olioon viivan. Palauttaa viitteen listtyyn
     * viivaan, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *     space.addLine(0, 0, 0, 100, 100, 100).setColor(Color.RED);
     * </pre>
     * @param x1 lhtpisteen x-koordinaatti
     * @param y1 lhtpisteen y-koordinaatti
     * @param z1 lhtpisteen z-koordinaatti
     * @param x2 loppupisteen x-koordinaatti
     * @param y2 loppupisteen y-koordinaatti
     * @param z2 loppupisteen z-koordinaatti
     * @return listty Line-olio
     */
    Line addLine(double x1, double y1, double z1, double x2, double y2,
            double z2);


    /**
     * Lis olioon monikulmion. Palauttaa viitteen listtyyn
     * monikulmioon, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *      double[] xpoints = {-1, 2, 4};
     *      double[] ypoints = [5, 6, 7};
     *      space.addPolygon(xpoints, ypoints).setColor(Color.RED);
     * </pre>
     * @param xpoints monikulmion krkipisteiden x-koordinaatit
     * @param ypoints monikulmion kskipisteiden y-koordinaatit
     * @return listty Polygon-olio
     */
    Polygon addPolygon(double[] xpoints, double[] ypoints);


    /**
     * Lis olioon monikulmion. Palauttaa viitteen listtyyn
     * monikulmioon, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *      double[][] points = {{2, 6}, {5, 5}, {-1, 2.5}}
     *      space.addPoylgon(points).setColor(Color.RED);
     * </pre>
     * @param points monikulmion krkipisteiden koordinaatit
     * @return listty Polygon-olio
     */
    Polygon addPolygon(double[][] points);


    /**
     * Lis olioon tytetyn monikulmion. Palauttaa viitteen listtyyn
     * tytettyyn monikulmioon, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *      double[] xpoints = {-1, 2, 4};
     *      double[] ypoints = [5, 6, 7};
     *      space.addFillPolygon(xpoints, ypoints).setColor(Color.RED);
     * </pre>
     * @param xpoints monikulmion krkipisteiden x-koordinaatit
     * @param ypoints monikulmion kskipisteiden y-koordinaatit
     * @return listty FillPolygon-olio
     */
    FillPolygon addFillPolygon(double[] xpoints, double[] ypoints);


    /**
     * Lis olioon tytetyn monikulmion. Palauttaa viitteen listtyyn
     * tytettyyn monikulmioon, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *      double[][] points = {{2, 6}, {5, 5}, {-1, 2.5}}
     *      space.addFillPoylgon(points).setColor(Color.RED);
     * </pre>
     * @param points monikulmion krkipisteiden koordinaatit
     * @return listty FillPolygon-olio
     */
    FillPolygon addFillPolygon(double[][] points);


    /**
     * Lis olioon ympyrn. Palauttaa viitteen listtyyn
     * ympyrn, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *       space.addCircle(0, 0, 1).setColor(Color.RED);
     * </pre>
     * @param x ympyrn keskipisteen x-koordinaatti
     * @param y ympyrn keskipisteen y-koordinaatti
     * @param r ympyrn sde
     * @return listty Circle-olio
     */
    Circle addCircle(double x, double y, double r);


    /**
     * Lis olioon ympyrn. Palauttaa viitteen listtyyn
     * ympyrn, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *      space.addCircle(0, 0, 0, 1).setColor(Color.RED);
     * </pre>
     * @param x ympyrn keskipisteen x-koordinaatti
     * @param y ympyrn keskipisteen y-koordinaatti
     * @param z ympyrn keskipisteen z-koordinaatti
     * @param r ympyrn sde
     * @return listty Circle-olio
     */
    Circle addCircle(double x, double y, double z, double r);


    /**
     * Lis olioon koordinaatistoakselit. Palauttaa viitteen listtyyn
     * akselistoon, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *      space.addAxis(5, 5, 5).setColor(Color.RED);
     * </pre>
     * @param xLength x-akselin pituus
     * @param yLength y-akselin pituus
     * @param zLength z-akselin pituus
     * @return listty Axis-olio
     */
    Axis addAxis(double xLength, double yLength, double zLength);


    /**
     * Lis olioon koordinaatistoakselit. Palauttaa viitteen listtyyn
     * akselistoon, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *     space.addAxis(5, 5, 5, 0, 0, 0).setColor(Color.RED);
     * </pre>
     * @param xLength x-akselin pituus
     * @param yLength y-akselin pituus
     * @param zLength z-akselin pituus
     * @param x akselien leikkauspisteen x-koordinaatti
     * @param y akselien leikkauspisteen y-koordinaatti
     * @param z akselien leikkauspisteen z-koordinaatti
     * @return listty Axis-olio
     */
    Axis addAxis(double xLength, double yLength, double zLength, double x,
            double y, double z);


    /**
     * Lis olioon kuvion joka on Drawable-olio. Palauttaa viitteen listtyyn
     * kuvioon, jolloin sille voidaan heti suorittaa jokin muukin operaatio, esimerkiksi:
     * <pre>
     *      space.addShape(new Line(0, 0, 10, 10)).setColor(Color.RED);
     * </pre>
     * @param shape listtv kuvio
     * @return listty olio
     */
    Drawable addShape(Drawable shape);


    /**
     * Listtn funktion kuvaaja  R:lta R:ll
     * @param f  listtv funktio
     * @param xmin - x min
     * @param xmax - y min
     * @return listty funktio-olio
     */
    FunctionMapRR add(FunctionRR f, double xmin, double xmax);


    /**
     * Listtn funktion kuvaaja  R2:lta R:ll
     * @param f   listv funktio
     * @param xmin  x min
     * @param ymin  y min
     * @param xmax  x max
     * @param ymax  y max
     * @return listty funktio-olio
     */
    FunctionMapR2R add(FunctionR2R f, double xmin, double ymin, double xmax,
            double ymax);
}
