package fi.jyu.mit.graphics;

/**
 * Luokka jolla voi helposti luoda kuviota avaruuteen.
 * Avaruuteen voi list kuvioita, esimerkiksi
 * <pre>
 *      space.addLine(0,0,100,100).setColor(Color.RED);
 *      space.addCircle(0, 0, 1).setColor(Color.RED);
 *      space.addShape(new Line(0, 0, 10, 10)).setColor(Color.RED);
 * </pre>
 * @author Markus Kivioja
 *
 */
public class EasySpace extends Space implements Easy {

    /**
     * Luo uuden avaruuden
     *
     */
    public EasySpace() {
        super();
    }


    @Override
    public Drawable addShape(Drawable shape) {
        return add(shape);
    }


    @Override
    public Line addLine(double x1, double y1, double x2, double y2) {
        return (Line) add(new Line(x1, y1, x2, y2));
    }


    @Override
    public Line addLine(double x1, double y1, double z1, double x2, double y2,
            double z2) {
        return (Line) add(new Line(x1, y1, z1, x2, y2, z2));
    }


    @Override
    public Circle addCircle(double x, double y, double r) {
        return (Circle) add(new Circle(x, y, r));
    }


    @Override
    public Circle addCircle(double x, double y, double z, double r) {
        return (Circle) add(new Circle(x, y, z, r));
    }


    @Override
    public Polygon addPolygon(double[] xpoints, double[] ypoints) {
        return (Polygon) add(new Polygon(xpoints, ypoints));
    }


    @Override
    public Polygon addPolygon(double[][] points) {
        return (Polygon) add(new Polygon(points));
    }


    @Override
    public Axis addAxis(double xLength, double yLength, double zLength) {
        return (Axis) add(new Axis(xLength, yLength, zLength));
    }


    @Override
    public Axis addAxis(double xLength, double yLength, double zLength,
            double x, double y, double z) {
        return (Axis) add(new Axis(xLength, yLength, zLength, x, y, z));
    }


    @Override
    public FillPolygon addFillPolygon(double[] xpoints, double[] ypoints) {
        return (FillPolygon) add(new FillPolygon(xpoints, ypoints));
    }


    @Override
    public FillPolygon addFillPolygon(double[][] points) {
        return (FillPolygon) add(new FillPolygon(points));
    }


    @Override
    public FunctionMapRR add(FunctionRR f, double xmin, double xmax) {
        return (FunctionMapRR) add(new FunctionMapRR(f, xmin, xmax));
    }


    @Override
    public FunctionMapR2R add(FunctionR2R f, double xmin, double ymin,
            double xmax, double ymax) {
        return (FunctionMapR2R) add(
                new FunctionMapR2R(f, xmin, ymin, xmax, ymax));
    }
}
