package fi.jyu.mit.graphics;

import java.awt.Graphics;

/**
 * Luokka tytetyille monikulmioille
 * @author Markus Kivioja
 *
 */
public class FillPolygon extends Polygon {

    public FillPolygon() {
        super();
    }


    /**
     * Luo tytetyn monikulmion annetuista kulmapisteiden
     * x- ja y-koordinaateista
     * @param xpoints taulukko kulmapisteiden x-koordinaateista
     * @param ypoints taulukko kulmapisteiden y-koordinaateista
     */
    public FillPolygon(double[] xpoints, double[] ypoints) {
        super(xpoints, ypoints);
    }


    /**
     * Luo tytetyn monikulmion annetuista kulmapisteiden
     * x-, y- ja z-koordinaateista
     * @param xpoints taulukko kulmapisteiden x-koordinaateista
     * @param ypoints taulukko kulmapisteiden y-koordinaateista
     * @param zpoints taulikko kulmapisteiden z-koordinaateista
     */
    public FillPolygon(double[] xpoints, double[] ypoints, double[] zpoints) {
        super(xpoints, ypoints, zpoints);
    }


    /**
     * Luo tytetyn kaksi- tai kolmiulotteisen monikulmion annetuista kulmapisteist, esimerkiksi:
     * <pre>
     *      double[][] points = {{2, 6}, {5, 5}, {-1, 2.5}}
     *      FillPolygon fp = new FillPolygon(points);
     *      
     *      Luo tytetyn monikulmion jonka krkipisteet ovat (2, 6), (5, 5) ja (-1, 2.5)
     * </pre>
     * @param points 
     */
    public FillPolygon(double[][] points) {
        super(points);
    }


    /**
     * Luo tytetyn monikulmion annetuista kulmapisteist jotka ovat RPoint olioita
     * @param points taulukko kulmapisteist jotka ovat RPoint-olioita
     */
    public FillPolygon(RPoint[] points) {
        super(points);
    }


    public void drawEmpty(Graphics g, Matrix a) {
        super.drawShape(g, a);
    }


    @Override
    protected void drawShape(Graphics g, Matrix a) {
        int[][] sPoints = makeScreenPoints(a);
        g.fillPolygon(sPoints[0], sPoints[1], getCount());
    }
}
