package fi.jyu.mit.graphics;

import static java.lang.Math.round;

/**
 * Matriisliluokka kuvamuunnoslaskuihin. 
 * @author Markus Kivioja
 * @author Vesa Lappalainen
 *
 */
public class Matrix {

    protected final static int VSIZE = 4;
    private double mat[][] = new double[VSIZE][VSIZE];
    private Matrix inversion;


    /**
     * Luo uuden 4x4 yksikkmatriin
     */
    public Matrix() {
        this.ident();
    }


    /**
     * Luo kopion matriisista M
     * @param M matriisi josta kopio luodaan
     */
    public Matrix(Matrix M) {
        for (int r = 0; r < VSIZE; r++)
            for (int s = 0; s < VSIZE; s++)
                mat[r][s] = M.mat[r][s];
    }


    /**
     * Muuttaa matriisin yksikkmatriisiksi
     */
    protected final void ident() {
        this.inversion = null;
        for (int r = 0; r < VSIZE; r++)
            for (int s = 0; s < VSIZE; s++)
                mat[r][s] = 0.0;
        for (int r = 0; r < VSIZE; r++)
            mat[r][r] = 1.0;
    }


    /**
     * Palauttaa matriisin r:n rivin kopion
     * @param r palautettava rivi
     * @return r:n rivin kopio
     */
    public Vector row(int r) { // NOPMD
        Vector v = new Vector(); // NOPMD
        for (int s = 0; s < VSIZE; s++) {
            v.set(s, mat[r][s]);
        }
        return v;
    }


    /**
     * Palauttaa matriisin s:n sarakkeen kopion
     * @param s palautettava sarake
     * @return s:n sarakkeen kopio
     */
    public Vector column(int s) { // NOPMD
        Vector v = new Vector(); // NOPMD
        for (int r = 0; r < VSIZE; r++) {
            v.set(r, mat[r][s]);
        }
        return v;
    }


    /**
     * Asettaa tmn matriisin annetuksi riviksi annetun vektorin
     * @param r muutettava rivi
     * @param v vektori joka asetetaan tmn matriisin riviksi
     */
    public void setRow(int r, Vector v) {
        this.inversion = null;
        for (int i = 0; i < VSIZE; i++)
            mat[r][i] = v.a(i);
    }


    /**
     * Asettaa tmn matriisin annetuksi sarakkeeksi annetun vektorin
     * @param s muutettava sarake
     * @param v vektori joka asetetaan tmn matriisin sarakkeeksi
     */
    public void setColumn(int s, Vector v) {
        this.inversion = null;
        for (int i = 0; i < VSIZE; i++)
            mat[i][s] = v.a(i);
    }


    /**
     * Palauttaa matriisin alkion paikassa r,s
     * @param r rivi jolta alkio halutaan
     * @param s sarake jolta alkio halutaan
     * @return r,s paikassa oleva alkio
     */
    public double m(int r, int s) { // NOPMD
        return mat[r][s];
    }


    /**
     * Asettaa matriisin paikassa r,s olevan alkion
     * @param r rivi jonka alkio asetetaan
     * @param s sarake jonka alkio asetetaan
     * @param d uusi arvo alkiolle
     */
    protected void set(int r, int s, double d) {
        this.inversion = null;
        mat[r][s] = d;
    }


    /**
     * Kertoo matriisin b matriisilla ja palauttaa uuden matriisin tuloksena
     * @param b matriisi joka kerrotaan
     * @return uusi matriisi this * b
     */
    public Matrix multiply(Matrix b) {
        if (b == null)
            return this;
        return multiply(b, new Matrix());
    }


    /**
     * Kertoo matriisin b matriisilla ja palauttaa uuden matriisin tuloksena.
     * this eik b ei saa olla c!
     * @param b matriisi joka kerrotaan
     * @param c matriisi johon tulos tulee
     * @return uusi matriisi this * b
     */
    public Matrix multiply(Matrix b, Matrix c) {
        // if ( b == c ) throw new Execption
        for (int r = 0; r < VSIZE; r++)
            for (int s = 0; s < VSIZE; s++) {
                double sum = 0;
                for (int i = 0; i < VSIZE; i++)
                    sum += mat[r][i] * b.mat[i][s];
                c.mat[r][s] = sum;
            }
        return c;
    }


    /**
     * Kertoo matriisin b matriisilla ja laittaa tuloksen itseens
     * @param b matriisi joka kerrotaan
     * @return  this * b
     */
    public Matrix multiplyThis(Matrix b) {
        this.inversion = null;
        double row[] = { 0, 0, 0, 0 };
        for (int r = 0; r < VSIZE; r++) {
            for (int s = 0; s < VSIZE; s++) {
                double sum = 0;
                for (int i = 0; i < VSIZE; i++)
                    sum += mat[r][i] * b.mat[i][s];
                row[s] = sum;
            }
            for (int s = 0; s < VSIZE; s++)
                mat[r][s] = row[s];
        }
        return this;
    }


    /**
     * Kertoo matriisin b matriisilla ja laittaa tuloksen itseens
     * @param b matriisi joka kerrotaan
     * @param row lakun avuksi tuotava 4 paikkainen taulukko
     * @return  this * b
     */
    public Matrix multiplyThis(Matrix b, double row[]) {
        this.inversion = null;
        // double row[] = {0,0,0,0};
        for (int r = 0; r < VSIZE; r++) {
            for (int s = 0; s < VSIZE; s++) {
                double sum = 0;
                for (int i = 0; i < VSIZE; i++)
                    sum += mat[r][i] * b.mat[i][s];
                row[s] = sum;
            }
            for (int s = 0; s < VSIZE; s++)
                mat[r][s] = row[s];
        }
        return this;
    }


    /**
     * Kertoo vektorin tll matriisilla, luo uuden paluuvektorin
     * @param b vektori joka kerrotaan
     * @return this * b
     */
    public Vector multiply(Vector b) { // NOPMD
        Vector y = new Vector(); // NOPMD
        for (int r = 0; r < VSIZE; r++) {
            double sum = 0;
            for (int s = 0; s < VSIZE; s++)
                sum += mat[r][s] * b.a(s);
            y.set(r, sum);
        }
        return y;
    }


    /**
     * Kertoo vektorin tll matriisilla ja tuloksen vektoriin y
     * @param b vektori joka kerrotaan
     * @param y vektori johon tulos tulee
     * @return this * b
     */
    public Vector multiply(Vector b, Vector y) { // NOPMD
        for (int r = 0; r < VSIZE; r++) {
            double sum = 0;
            for (int s = 0; s < VSIZE; s++)
                sum += mat[r][s] * b.a(s);
            y.set(r, sum);
        }
        return y;
    }


    /**
     * Kertoo vektorin tll matriisilla ja sijoittaa tuloksen nyttkoordinaattiin iy.
     * Spesiaalikertominen erityisesti kuvamuunnoksiin, joissa
     * tarvitaan vain 2 ensimmist alkiota vektorista iy.
     * @param b vektori joka kerrotaan
     * @param iy vektori johon tulos tulee
     * @return this * b
     */
    public SPoint transform(Vector b, SPoint iy) { // NOPMD
        double sum0 = 0, sum1 = 0;
        for (int s = 0; s < VSIZE; s++)
            sum0 += mat[0][s] * b.a(s);
        for (int s = 0; s < VSIZE; s++)
            sum1 += mat[1][s] * b.a(s);
        // TODO: pitisi viel laskea viimeisen rivin arvo, jos
        // se on !=1; niin sill jaetaan x ja y.
        // Tarvitaan sitten kun mat[3] rivill on jotakin
        return iy.set((int) round(sum0), (int) round(sum1));
    }


    /**
     * Kertoo vektorin tll matriisilla ja sijoittaa tuloksen nyttkoordinaattiin iy.
     * Spesiaalikertominen erityisesti kuvamuunnoksiin, joissa
     * tarvitaan vain 2 ensimmist alkiota vektorista iy.
     * @param b vektori joka kerrotaan
     * @param iy vektori johon tulos tulee
     * @return this * b
     */
    public RPoint transform(Vector b, RPoint iy) { // NOPMD
        double sum0 = 0, sum1 = 0;
        for (int s = 0; s < VSIZE; s++)
            sum0 += mat[0][s] * b.a(s);
        for (int s = 0; s < VSIZE; s++)
            sum1 += mat[1][s] * b.a(s);
        // TODO: pitisi viel laskea viimeisen rivin arvo, jos
        // se on !=1; niin sill jaetaan x ja y.
        // Tarvitaan sitten kun mat[3] rivill on jotakin
        iy.set(sum0, sum1, 0);
        return iy;
    }


    /**
     * Kertoo vektorin tll matriisilla ja sijoittaa tuloksen nyttkoordinaattiin iy.
     * Spesiaalikertominen erityisesti kuvamuunnoksiin, joissa
     * tarvitaan vain 2 ensimmist alkiota vektorista iy.
     * @param b vektori joka kerrotaan
     * @param iy vektori johon tulos tulee
     * @return this * b
     */
    public Vector transform(Vector b, Vector iy) { // NOPMD
        double sum0 = 0, sum1 = 0;
        for (int s = 0; s < VSIZE; s++)
            sum0 += mat[0][s] * b.a(s);
        for (int s = 0; s < VSIZE; s++)
            sum1 += mat[1][s] * b.a(s);
        // TODO: pitisi viel laskea viimeisen rivin arvo, jos
        // se on !=1; niin sill jaetaan x ja y.
        // Tarvitaan sitten kun mat[3] rivill on jotakin
        return iy.set(sum0, sum1);
    }


    /**
     * Muuntaa koordinaatin (x,y,z) nytn pisteeksi. 
     * @param x
     * @param y
     * @param z
     * @return uusi nytn piste joka edustaa (x,y,z)
     */
    public SPoint transform(double x, double y, double z) {
        Vector vr = new Vector(x, y, z); // NOPMD
        SPoint sp = new SPoint(0, 0);
        return this.transform(vr, sp);
    }


    /**
     * Muuntaa koordinaatin (x,y,z) nytn pisteeksi.
     * "Optimoitu" versio, jolle viedn tytilat parametrina. 
     * @param x
     * @param y
     * @param z
     * @param vr vektori johon saa sijoittaa vliaikaisesti (x,y,z)
     * @param sp nytn piste johon tuloksen saa sijoittaa.
     * @return parametrina tuotu nytn piste joka edustaa (x,y,z)
     */
    public SPoint transform(double x, double y, double z, Vector vr,
            SPoint sp) { // NOPMD
        return this.transform(vr.set(x, y, z), sp);
    }


    /**
     * Muuntaa koordinaatin (x,y,z) nytn pisteeksi.
     * "Optimoitu" versio, jolle viedn tytilat parametrina. 
     * @param x
     * @param y
     * @param z
     * @param vr vektori johon saa sijoittaa vliaikaisesti (x,y,z)
     * @param sp nytn piste johon tuloksen saa sijoittaa.
     * @return parametrina tuotu nytn piste joka edustaa (x,y,z)
     */
    public Vector transform(double x, double y, double z, Vector vr,
            Vector sp) { // NOPMD
        return this.transform(vr.set(x, y, z), sp);
    }


    /**
     * Muuntaa pisteen p nytn pisteeksi. 
     * @param p muunettava piste.
     * @return uusi nytnpiste joka vastaa pistett p.
     */
    public SPoint transform(RPoint p) {
        SPoint sp = new SPoint(0, 0);
        return this.transform(p, sp);
    }


    /**
     * Muuntaa pisteen p nytn pisteeksi. 
     * @param p muunettava piste.
     * @return uusi nytnpiste joka vastaa pistett p.
     */
    public RPoint transformR(RPoint p) {
        RPoint rp = new RPoint(0, 0);
        return this.transform(p, rp);
    }


    /**
     * Laskee tmn matriisin knteismatriisin Gaussin 
     * eliminointimenetelmll ja palauttaa sen.
     * (Menetelm on hidas suurilla VSIZE-muuttujan arvoilla)
     * @return inversion, tmn matriisin knteismatriisi
     */
    public Matrix getInversion() {
        if (this.inversion == null) {
            Matrix temp = new Matrix(this);
            Matrix tempInversion = new Matrix();
            double multiplier = 0;

            // Nollataan diagonaalin alapuolella olevat alkiot
            for (int i = 0; i < VSIZE; i++) {
                for (int j = i + 1; j < VSIZE; j++) {
                    multiplier = -(temp.m(j, i) / temp.m(i, i));
                    temp.setRow(j, temp.row(j)
                            .sum(temp.row(i).scalarProduct(multiplier)));
                    tempInversion.setRow(j, tempInversion.row(j).sum(
                            tempInversion.row(i).scalarProduct(multiplier)));
                }
            }

            // Nollataan diagonaalin ylpuolella olevat alkiot
            for (int i = VSIZE - 1; i >= 0; i--) {
                for (int j = i - 1; j >= 0; j--) {
                    multiplier = -(temp.m(j, i) / temp.m(i, i));
                    temp.setRow(j, temp.row(j)
                            .sum(temp.row(i).scalarProduct(multiplier)));
                    tempInversion.setRow(j, tempInversion.row(j).sum(
                            tempInversion.row(i).scalarProduct(multiplier)));
                }
            }

            // Jaetaan rivit diagonaalialkioilla
            for (int i = 0; i < VSIZE; i++) {
                multiplier = 1 / temp.m(i, i);
                tempInversion.setRow(i,
                        tempInversion.row(i).scalarProduct(multiplier));
            }
            this.inversion = tempInversion;
        }
        return this.inversion;
    }


    /**
     * Palauttaa tmn matrisiin merkkijonona
     * @return tm matriisi merkkijonona
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int r = 0; r < VSIZE; r++) {
            sb.append("( ");
            for (int s = 0; s < VSIZE; s++)
                sb.append(" " + mat[r][s]);
            sb.append(" )\n");
        }
        return sb.toString();
    }

}
