package fi.jyu.mit.graphics;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * Luokka jolla voidaan siirt kuvia hiiren klikkauksen kohdalle.
 * TODO: ei (ehk) toimi jos paneelia on skaalattu.  Tarvitsee toteuttaa
 * ensin knteiskuvaus.  Samoin ei toimi jos peruspolkua on muutettu. 
 * @author Vesa Lappalainen
 * @version 20.8.2008
 * @version 8.3.2012
 */
public class MouseFollower extends MouseAdapter {
    public interface MouseMoveFollower {
        public void mouseMoved(RPoint rp);
    }

    private Transformable object;
    private MouseMoveFollower follower = null;
    private SpacePanel panel;
    private boolean currentDrag = false;
    private boolean currentMove = false;
    private boolean currentPressed = true;


    /**
     * Alustetaan hiiren seuraaja.
     * @param panel    paneli jossa tapahtumia muutoksia seurataan
     * @param object   olio jota siirretn hiiren mukaan.
     * @param x olion koordinaatti johon keskitetn 
     * @param y olion koordinaatti johon keskitetn
     * @param z olion koordinaatti johon keskitetn
     * @param drag liikutetaan
     */
    @SuppressWarnings("unused")
    public MouseFollower(SpacePanel panel, Transformable object, double x,
            double y, double z, boolean drag) { // NOPMD
        this.object = object;
        this.panel = panel;
        panel.addMouseListener(this);
        startFollow(drag);
    }


    /**
     * Alustetaan hiiren seuraaja.
     * @param panel    paneli jossa tapahtumia muutoksia seurataan
     * @param follower ksittelij jota kutsutaan kun hiiri liikahtaa
     * @param drag jos true, kutsutaan hiiri pohjassa liikuttelusta, muuten vain klikkauksesta
     */
    public MouseFollower(SpacePanel panel, MouseMoveFollower follower,
            boolean drag) {
        this.follower = follower;
        this.panel = panel;
        startFollow(drag);
    }


    /**
     * Alustetaan hiiren seuraaja siten, ett z-koordinaatin arvo on nolla ja liikutetaan on tosi
     * @param panel paneeli jossa tapahtumia muutoksia seurataan
     * @param object olio jota siirretn hiiren mukaan.
     * @param x olion koordinaatti johon keskitetn 
     * @param y olion koordinaatti johon keskitetn
     */
    public MouseFollower(SpacePanel panel, Transformable object, double x,
            double y) {
        this(panel, object, x, y, 0, true);
    }


    /**
     * Alustetaan hiiren seuraaja siten, ett z-koordinaatin arvo on nolla ja liikutetaan on tosi
     * @param panel paneeli jossa tapahtumia muutoksia seurataan
     * @param object olio jota siirretn hiiren mukaan.
     * @param drag jos true, kutsutaan hiiri pohjassa liikuttelusta, muuten vain klikkauksesta
     */
    public MouseFollower(SpacePanel panel, Transformable object, boolean drag) {
        this(panel, object, 0, 0, 0, drag);
    }


    /**
     * Alustetaan hiiren seuraaja siten, ett z-koordinaatin arvo on nolla ja liikutetaan on tosi
     * @param panel paneeli jossa tapahtumia muutoksia seurataan
     * @param object olio jota siirretn hiiren mukaan.
     */
    public MouseFollower(SpacePanel panel, Transformable object) {
        this(panel, object, 0, 0, 0, false);
    }


    /**
     * Lopetetaan hiiren seuraaminen
     */
    public void stopFollow() {
        panel.removeMouseListener(this);
        panel.removeMouseMotionListener(this);
    }


    /**
     * Aloitetaan hiiren seuraaminen
     * @param drag jos true, kutsutaan hiiri pohjassa liikuttelusta, muuten vain klikkauksesta
     */
    public void startFollow(boolean drag) {
        stopFollow();
        panel.addMouseListener(this);
        panel.addMouseMotionListener(this);
        currentDrag = drag;
    }


    /**
     * Aloitetaan hiiren seuraaminen
     */
    public void startFollow() {
        startFollow(currentDrag);
    }


    /**
     * Palautetaan panelin ruutpistett vastaava reaalimaailman piste tasosta z=0
     * @param panel mhink paneelin piste
     * @param x panelin ruutupisteen x-koordinaatti
     * @param y panelin ruutupisteen x-koordinaatti
     * @param sp piste johon palautetaan klikatuin pisteen laitekoordinaatit
     * @return reaalimaailman piste 
     */
    public static RPoint getRPoint(SpacePanel panel, double x, double y,
            RPoint sp) {
        RPoint rp = new RPoint();
        panel.getFullTransform().getInversion().transform(x, y, 0, sp, rp);
        return rp;
    }


    /**
     * Palautetaan panelin ruutpistett vastaava reaalimaailman piste tasosta z=0
     * @param panel mhink paneelin piste
     * @param x panelin ruutupisteen x-koordinaatti
     * @param y panelin ruutupisteen x-koordinaatti
     * @return reaalimaailman piste 
     */
    public static RPoint getRPoint(SpacePanel panel, double x, double y) {
        return getRPoint(panel, x, y, new RPoint());
    }


    /**
     * Palautetaan mouse eventti vastaava reaalimaailman piste tasosta z=0
     * @param panel mhink paneelin piste
     * @param e hiiren tapahtuman tiedot
     * @param sp piste johon palautetaan klikatuin pisteen laitekoordinaatit
     * @return reaalimaailman piste 
     */
    public static RPoint getRPoint(SpacePanel panel, MouseEvent e, RPoint sp) {
        return getRPoint(panel, e.getX(), e.getY(), sp);
    }


    /**
     * Palautetaan mouse eventti vastaava reaalimaailman piste tasosta z=0
     * @param panel mhink paneelin piste
     * @param e hiiren tapahtuman tiedot
     * @return reaalimaailman piste 
     */
    public static RPoint getRPoint(SpacePanel panel, MouseEvent e) {
        return getRPoint(panel, e, new RPoint());
    }


    /**
     * This is only for inner purposes
     * @param e MouseEvent
     */
    private void doMove(MouseEvent e) {
        // if ( ! (e.getComponent() instanceof SpacePanel) ) return;
        SpacePanel panel = (SpacePanel) e.getComponent();
        RPoint sp = new RPoint();
        if (object != null) {
            BasicShape comp = (BasicShape) object;
            SPoint op = comp.getPoint(panel.getFullTransform(), 0, 0, 0);
            RPoint rp = getRPoint(panel, e.getX() - op.getX(),
                    e.getY() - op.getY(), sp);
            RPoint rp0 = getRPoint(panel, 0, 0, sp);
            // comp.setTransform(new
            // TranslateMatrix(sp.getX()-op.getX(),sp.getY()-op.getY(),0).multiply(comp.getTransform(null)));
            comp.setTransform(new TranslateMatrix(rp.getX() - rp0.getX(),
                    rp.getY() - rp0.getY(), 0)
                            .multiply(comp.getTransform(null)));
        }
        if (follower != null) {
            RPoint rp = getRPoint(panel, e, sp);
            follower.mouseMoved(rp);
        }
    }


    /**
     * Hiiren painalluksen tapahtumanksittelij
     * @param e ksiteltv hiiritapahtuma
     */
    @Override
    public void mousePressed(MouseEvent e) {
        if (currentPressed)
            doMove(e);
    }


    /**
     * Hiiren raahauksen tapahtumanksittelij
     * @param e ksiteltv hiiritapahtuma
     */
    @Override
    public void mouseDragged(MouseEvent e) {
        if (currentDrag)
            doMove(e);
    }


    /**
     * Tyhj toteutus metodille. MouseAdapter Java 6:ssa sislt tyhjt
     * toteutukset MouseMotionListener-rajapinnan metodeille. Javan versiot
     * 1.5:st aiempiin eivt nit sisll.
     * 
     * @param e ei kytet.
     * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
     */
    @Override
    public void mouseMoved(MouseEvent e) {
        if (currentMove)
            doMove(e);
    }


    /**
     * @return seurataanko liikuttelua
     */
    public boolean isMove() {
        return currentMove;
    }


    /**
     * Asetetaan seuraamaan liikuttelua
     * @param move plle vaiko pois
     */
    public void setMove(boolean move) {
        this.currentMove = move;
    }


    /**
     * @return Seurataanko clikkausta
     */
    public boolean isPressed() {
        return currentPressed;
    }


    /**
     * Asetetaan seuraamaan klikkausta
     * @param pressed plle vaiko pois
     */
    public void setPressed(boolean pressed) {
        this.currentPressed = pressed;
    }


    /**
     * @return seurataanko raahaamista
     */
    public boolean isDrag() {
        return currentDrag;
    }


    /**
     * Asetetaan seuraamaan raahaamista 
     * @param drag
     */
    public void setDrag(boolean drag) {
        this.currentDrag = drag;
    }
}
