package fi.jyu.mit.graphics;

/**
 * Neliskulmio jonka krkipisteet kolmiulotteisia pisteit
 * @author Markus Kivioja
 *
 */
public class RRect {

    private final RPoint p1, p2;


    /**
     * Luo uuden neliskulmion jonka krkipisteet ovat RPoint-olioita
     * @param leftTop vasen ylkulma
     * @param rightBottom oikea alakulma
     */
    public RRect(RPoint leftTop, RPoint rightBottom) {
        this.p1 = leftTop;
        this.p2 = rightBottom;
    }


    /**
     * Luo uuden annettujen krkipisteiden koordinaattien mukaisen neliskulmion
     * @param left vaseman reunan x-koordinaatti
     * @param top ylreunan y-koordinaatti
     * @param right oikean reunan x-koordinaatti
     * @param bottom alareunan y-koordinaatti
     */
    public RRect(double left, double top, double right, double bottom) {
        this.p1 = new RPoint(left, top);
        this.p2 = new RPoint(right, bottom);
    }


    /**
     * Antaa tmn neliskulmion leveyden
     * @return neliskulmion leveys
     */
    public double width() {
        return p2.getX() - p1.getX();
    }


    /**
     * Antaa tmn neliskulmion korkeuden
     * @return neliskulmion korkeus
     */
    public double height() {
        return p2.getY() - p1.getY();
    }


    /**
     * Antaa tmn neliskulmion keskipisteen x-koordinaatin
     * @return neliskulmion keskipisteen x-koordinaatti
     */
    public double midX() {
        return (p1.getX() + p2.getX()) / 2.0;
    }


    /**
     * Antaa tmn neliskulmion keskipisteen y-koordinaatin
     * @return neliskulmion keskipisteen y-koordinaatti
     */
    public double midY() {
        return (p1.getY() + p2.getY()) / 2.0;
    }


    /**
     * Antaa tmn neliskulmion vasemman ylkulman RPoint-oliona
     * @return neliskulmion vasen ylkulma
     */
    public RPoint getP1() {
        return p1;
    }


    /**
     * Antaa tmn neliskulmion oikean alakulman RPoint-oliona
     * @return neliskulmion oikea alakulma
     */
    public RPoint getP2() {
        return p2;
    }
}
