package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Avaruus jossa listyt kuviot ovat.
 * Avaruuteen voi list kuvioita, esimerkiksi:
 * <pre>
 *      space.add(new Line(0, 0, 1, 1)).setColor(255, 255, 0);
 *      space.add(new Circle(2, 4, 9)).rotate(Axis.Y, 94);
 * </pre>
 * @author Markus Kivioja
 *
 */
public class Space {

    private final DrawableCollection objects;
    private SpaceParent[] windows;
    private int numOfWins;


    /**
     * Luo uuden avaruuden
     */
    public Space() {
        this.objects = new DrawableCollection();
        this.objects.registerSpace(this);
        this.windows = new SpaceParent[20];
        this.numOfWins = 0;
    }


    /**
     * Rekisteri annetun ikkunan nyttmn tt avaruutta
     * @param window ikkuna joka nytt tt avaruutta
     */
    public void registerWindow(SpaceParent window) {
        if (numOfWins == this.windows.length) {
            SpaceParent[] temp = this.windows;
            this.windows = new SpaceParent[this.numOfWins + 20];
            for (int i = 0; i < temp.length; i++)
                this.windows[i] = temp[i];
        }
        this.windows[numOfWins] = window;
        numOfWins++;
    }


    /**
     * Poistaa annetun ikkunan tmn avaruuden rekisterist, kutsutaan
     * kun ikkuna lopettaa tmn avaruuden nyttmisen
     * @param window
     */
    public void unRegisterWindow(Window window) {
        int index = this.numOfWins;
        for (int i = 0; i < this.numOfWins; i++) {
            if (this.windows[i] == (window)) {
                index = i;
            }
        }
        if (index == this.numOfWins)
            return;
        for (int i = index; i < (this.numOfWins - 1); i++) {
            this.windows[i] = this.windows[i + 1];
        }
        this.windows[this.numOfWins - 1] = null; // NOPMD, vapautetaan muistia
        this.numOfWins--;
    }


    /**
     * Antaa Drawable-olion joka muodostuu kaikista 
     * tmn avaruuden sisltmist kuvioista
     * @return kokoelma tmn paneelin nyttmist kuvioista
     */
    public Drawable getSavedPath() {
        return this.objects;
    }


    /**
     * Piirt tmn avaruuden sisltmt kuviot uudelleen
     */
    public void redraw() {
        for (int i = 0; i < this.numOfWins; i++)
            this.windows[i].redraw();
    }


    /**
     * Piirt tmn avaruuden sisltmt kuviot annetulla matriisilla ja Graphics-oliolla
     * @param g Graphics-olio
     * @param a muunnosmatriisi
     */
    public void draw(Graphics g, Matrix a) {
        this.objects.draw(g, a);
    }


    /**
     * Lis avaruuteen kuvion joka on Drawable-olio ja palauttaa sen,
     * esimerkiksi:
     * <pre>
     *      space.add(new Line(0, 0, 1, 1)).setColor(255, 255, 0);
     * </pre>
     * @param shape listtv kuvio
     * @return listty kuvio
     */
    public Drawable add(Drawable shape) {
        objects.add(shape);
        redraw();
        return shape;
    }


    /**
     * Poistaa avaruudesta kuvion joka on Drawable-olio
     * @param shape poistettava kuvio
     */
    public void remove(Drawable shape) {
        objects.remove(shape);
        redraw();
    }


    /**
     * Poistaa avaruudesta kaikki kuviot
     *
     */
    public void removeAll() {
        objects.removeAll();
        redraw();
    }
}
