package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Luokka ympyrille
 * @author Markus Kivioja
 *
 */
public class Circle extends BasicShape {

    private final RPoint center;
    private double r;
    private int resolution;
    private int[] sXPoints;
    private int[] sYPoints;


    /**
     * Luo uuden ympyrn
     * @param x ympyrn keskipisteen x-koordinaatti
     * @param y ympyrn keskipisteen y-koordinaatti
     * @param r ympyrn sde
     */
    public Circle(double x, double y, double r) {
        super();
        this.center = new RPoint(x, y);
        this.r = r;
        initialize();
    }


    /**
     * Luo uuden ympyrn
     * @param x ympyrn keskipisteen x-koordinaatti
     * @param y ympyrn keskipisteen y-koordinaatti
     * @param z ympyrn keskipisteen z-koordinaatti
     * @param r ympyrn sde
     */
    public Circle(double x, double y, double z, double r) {
        super();
        this.center = new RPoint(x, y, z);
        this.r = r;
        initialize();
    }


    /**
     * Alustaa ympyrn asetukset
     */
    private void initialize() {
        this.resolution = 360;
        this.sXPoints = new int[resolution];
        this.sYPoints = new int[resolution];
    }


    /**
     * Asetetaan ympyrlle uusi sde
     * @param r uusi steen arvo
     * @return tm olio ketjuttamista varten
     */
    public Circle setR(double r) {
        this.r = r;
        redraw();
        return this;
    }


    /**
     * Piirretn ympyr
     * @see fi.jyu.mit.graphics.BasicShape#drawShape(java.awt.Graphics, fi.jyu.mit.graphics.Matrix)
     */
    @Override
    protected void drawShape(Graphics g, Matrix a) {
        Vector rp = new Vector(); 
        SPoint sp = new SPoint(0, 0);
        for (int i = 0; i < resolution; i++) {
            a.transform(
                    this.r * Math.cos((2 * Math.PI / resolution) * i)
                            + this.center.getX(),
                    this.r * Math.sin((2 * Math.PI / resolution) * i)
                            + this.center.getY(),
                    this.center.getZ(), rp, sp);
            sXPoints[i] = sp.getX();
            sYPoints[i] = sp.getY();
        }
        g.drawPolygon(sXPoints, sYPoints, resolution);
    }
}
