package fi.jyu.mit.graphics;

import java.awt.Graphics;

/**
 * Luokka R->R funktion kuvaajan piirtmiseksi.
 * @author Vesa Lappalainen
 *
 */
public class FunctionMapR2R extends BasicShape {
    private double xmin, ymin, xmax, ymax;
    private int nx, ny;
    private FunctionR2R f;


    /**
     * Luo uuden R->R funktion kuvaajan
     * @param f piirrettv funktio
     * @param xmin x minimi
     * @param ymin y minimi
     * @param xmax x maksimi
     * @param ymax y maksimi
     * @param nx piirrettvien x-pisteiden mr
     * @param ny piirrettvien y-pisteiden mr
     */
    public FunctionMapR2R(FunctionR2R f, double xmin, double ymin, double xmax,
            double ymax, int nx, int ny) {
        super();
        this.f = f;
        this.xmin = xmin;
        this.ymin = ymin;
        this.xmax = xmax;
        this.ymax = ymax;
        this.nx = nx;
        this.ny = ny;
    }


    /**
     * Luo uuden R->R funktion kuvaajan
     * @param f piirrettv funktio
     * @param xmin x minimi
     * @param ymin y minimi
     * @param xmax x maksimi
     * @param ymax y maksimi
     */
    public FunctionMapR2R(FunctionR2R f, double xmin, double ymin, double xmax,
            double ymax) {
        this(f, xmin, ymin, xmax, ymax, 40, 40);
    }


    @Override
    protected void drawShape(Graphics g, Matrix a) {
        double dx = (xmax - xmin) / nx;
        double dy = (ymax - ymin) / ny;
        double x, x2 = xmax - dx / 2;
        double y, y2 = ymax - dy / 2;
        SPoint sp = new SPoint(0, 0);
        Vector vr = new Vector();  

        y = ymin;
        while (y < y2) {
            xDir(g, a, y, vr, sp);
            y = y + dy;
        }
        xDir(g, a, ymax, vr, sp);

        x = xmin;
        while (x < x2) {
            yDir(g, a, x, vr, sp);
            x = x + dx;
        }
        yDir(g, a, xmax, vr, sp);
    }


    /**
     * This is only for inner purposes
     * @param g
     * @param a
     * @param y
     * @param vr
     * @param sp
     */
    private void xDir(Graphics g, Matrix a, double y, Vector vr, SPoint sp) { 
                                                                              // luulee
                                                                              // vrksi
                                                                              // vektoriksi
        double x = xmin;
        double dx = (xmax - xmin) / nx;
        int x1, y1, x2, y2;
        a.transform(vr.set(x, y, f.f(x, y)), sp);
        x1 = sp.getX();
        y1 = sp.getY();

        x += dx;
        while (x < xmax) {
            a.transform(vr.set(x, y, f.f(x, y)), sp);
            x2 = sp.getX();
            y2 = sp.getY();
            g.drawLine(x1, y1, x2, y2);
            x1 = x2;
            y1 = y2;
            x += dx;
        }
        x = xmax;
        a.transform(vr.set(x, y, f.f(x, y)), sp);
        x2 = sp.getX();
        y2 = sp.getY();
        g.drawLine(x1, y1, x2, y2);
    }


    /**
     * This is only for inner purposes
     * @param g
     * @param a
     * @param x
     * @param vr
     * @param sp
     */
    private void yDir(Graphics g, Matrix a, double x, Vector vr, SPoint sp) { 
                                                                              // Vector
        double y = ymin;
        double dy = (ymax - ymin) / ny;
        int x1, y1, x2, y2;
        a.transform(vr.set(x, y, f.f(x, y)), sp);
        x1 = sp.getX();
        y1 = sp.getY();

        y += dy;
        while (y < ymax) {
            a.transform(vr.set(x, y, f.f(x, y)), sp);
            x2 = sp.getX();
            y2 = sp.getY();
            g.drawLine(x1, y1, x2, y2);
            x1 = x2;
            y1 = y2;
            y += dy;
        }
        y = ymax;
        a.transform(vr.set(x, y, f.f(x, y)), sp);
        x2 = sp.getX();
        y2 = sp.getY();
        g.drawLine(x1, y1, x2, y2);
    }
}
