package sample;

import java.awt.Container;
import java.awt.EventQueue;
import java.awt.Rectangle;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;


import fi.jyu.mit.graphics.Window;
import javafx.stage.Stage;

/**
 * Ajetaan parametrina tuodun luokan main ja yritet��n ottaa
 * kuva ikkunasta
 * @author vesal
 * @version 30.1.2015
 */
public class Runner {
    /** 
     * Run needed class
     * @param args
     *  pakgace.classname for program to run,
     *  --captureName nameForPng.png
     *  --delay howLongToWaitBeforeCapture
     *  --rect x,y,width,height  // width==0 => capture whole screen
     *  
     */
    public static void main(String[] args) {
        try {
            Class<?> cla = Class.forName(args[0]);
            final Method method = cla.getMethod("main", String[].class);
            ArrayList<String> argl = new ArrayList<String>(Arrays.asList(args));
            argl.remove(0);
            String capturename = "capture.png";
            Rectangle rect = null;
            int i;
            int delay = 0;

            i = argl.indexOf("--captureName");
            if (i >= 0) {
                argl.remove(i);
                if (argl.size() > i) {
                    capturename = argl.get(i);
                    argl.remove(i);
                }
            }

            i = argl.indexOf("--delay");
            if (i >= 0) {
                argl.remove(i);
                if (argl.size() > i) {
                    delay = Integer.parseInt(argl.get(i));
                    argl.remove(i);
                }
            }

            i = argl.indexOf("--rect");
            if (i >= 0) {
                argl.remove(i);
                if (argl.size() > i) {
                    String[] rs = argl.get(i).split(",");
                    argl.remove(i);
                    int ri[] = new int[4];
                    for (int j = 0; j < 4; j++)
                        ri[j] = Integer.parseInt(rs[j]);
                    rect = new Rectangle(ri[0], ri[1], ri[2], ri[3]);
                }
            }

            boolean javaf = false;

            if (delay < 0) {
                delay = -delay;
                javaf = true;
            }
            final boolean isjavafx = javaf;

            final String[] ars = argl.toArray(new String[0]);

            if (isjavafx) {
                Thread t = new Thread() {
                    @Override
                    public void run() {
                        try {
                            method.invoke(null, (Object) ars);
                        } catch (Exception e) {
                            // System.err.println(e.toString());
                            e.printStackTrace();
                            System.exit(0);
                        }
                    }
                };
                t.start();
            } else
                method.invoke(null, (Object) ars);

            final String fcapturename = capturename;
            final Rectangle frect = rect;
            final int fdelay = delay;

            EventQueue.invokeLater(new Runnable() {
                @Override
                public void run() {
                    try {
                        Thread.sleep(fdelay);
                        if (frect == null) { // if we do not have rect, capture
                                             // window
                            Container w = Window.getLastOpenedWindow(); // try
                                                                        // if
                                                                        // graphics
                                                                        // has
                                                                        // window
                            if (w == null) // try to find open window
                                for (java.awt.Window wnd : java.awt.Window
                                        .getWindows()) {
                                    if (wnd.isShowing()) {
                                        w = wnd;
                                        break;
                                    }
                                }
                            if (w != null) { // if we have window, capture it
                                Window.capture(w, fcapturename);
                                System.exit(0);
                            }

                            javafx.stage.Window fxw = null;
                            if (isjavafx) {
                                //ObservableList<Stage> stages = FXRobotHelper
                                //        .getStages();
                                for (javafx.stage.Window wnd : javafx.stage.Window.getWindows()) {
                                    if (wnd.isShowing()) {
                                        fxw = wnd;
                                        break;
                                    }
                                }
                                
                                //if (stages.size() > 0) {
                                if (fxw != null ) {
                                    Stage s =  (Stage)fxw; //stages.get(0);
                                    Rectangle r = new Rectangle((int) s.getX(),
                                            (int) s.getY(), (int) s.getWidth(),
                                            (int) s.getHeight());
                                    // Rectangle r = new Rectangle(0, 0, 1920, 1200);
                                    Window.capture(r, fcapturename);
                                    System.exit(0);
                                }
                            }

                            Window.capture(fcapturename);
                            System.exit(0);
                        }
                        if ( frect != null) {
                            if (frect.width == 0)
                                Window.capture(fcapturename);
                            else
                                Window.capture(frect, fcapturename);
                        }
                        System.exit(0);
                    } catch (Exception e) {
                        System.err.println(e.getMessage());
                        System.exit(0);
                    }
                }
            });
        } catch (Exception e) {
            // System.err.println(e.toString());
            e.printStackTrace();
            System.exit(0);
        }
    }

}
